
#include "auth.h"
#include "debug.h"
#include "console_login.h"
#include "domain_name_cache.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <pthread.h>

char * crypt( const char *key, const char *salt );

static pthread_mutex_t crypt_lock = PTHREAD_MUTEX_INITIALIZER;

static int auth_userpass_assert( struct link *link, time_t stoptime )
{
	char line[AUTH_LINE_MAX];
	char name[AUTH_SUBJECT_MAX];
	char pass[AUTH_SUBJECT_MAX];
	char addr[LINK_ADDRESS_MAX];
	char service[DOMAIN_NAME_MAX];
	int port;
	int result=0;

	debug("auth_userpass: determining service name");

	if(link_address_remote(link,addr,&port)) {
		if(domain_name_cache_lookup_reverse(addr,service)) {
			/* use this service name */
		} else {
			/* just use the address */
			strcpy(service,addr);
		}
	} else {
		/* hmm, that's strange */
	}

	debug("auth_userpass: reading name and pass from user");

	if(console_login(service,name,sizeof(name),pass,sizeof(pass))) {
		sprintf(line,"%s\n%s\n",name,pass);
		debug("auth_userpass: sending name and pass to service");
		link_write(link,line,strlen(line),stoptime);

		debug("auth_userpass: clearing incore copies");

		memset(line,0,sizeof(line));
		memset(name,0,sizeof(name));
		memset(pass,0,sizeof(pass));

		debug("auth_userpass: getting response from service");

		if(link_readline(link,line,sizeof(line),stoptime)) {
			if(!strcmp(line,"yes")) {
				debug("auth_userpass: server accepts");
				result = 1;
			} else {
				debug("auth_userpass: server denies");
			}
		} else {
			debug("auth_userpass: lost connection");
		}
	} else {
		debug("auth_userpass: couldn't get password!");
	}

	return result;
}

static const char *passfile=0;

static int auth_userpass_lookup( const char *name, const char *pass )
{
	char line[AUTH_LINE_MAX];
	char tname[AUTH_SUBJECT_MAX];
	char tpass[AUTH_SUBJECT_MAX];
	int success=0;

	FILE *file = fopen(passfile,"r");
	if(file) {
		int linenum=0;
		while(fgets(line,sizeof(line),file)) {
			linenum++;
			if(sscanf(line,"%[^:]:%[^:\n]",tname,tpass)==2) {
				if(!strcmp(tname,name)) {
					pthread_mutex_lock(&crypt_lock);
					if(!strcmp(tpass,crypt(pass,tpass))) {
						success = 1;
					}
					pthread_mutex_unlock(&crypt_lock);
					break;
				}
			} else {
				debug("auth_userpass: bad data in %s at line %d",passfile,linenum);
			}
		}
		fclose(file);
	} else {
		debug("auth_userpass: couldn't open %s: %s",passfile,strerror(errno));
	}

	return success;
}

static int auth_userpass_accept( struct link *link, char **subject, time_t stoptime )
{
	char name[AUTH_SUBJECT_MAX];
	char pass[AUTH_SUBJECT_MAX];
	int success = 0;

	debug("auth_userpass: waiting for name and pass");

	if(link_readline(link,name,sizeof(name),stoptime) &&
	   link_readline(link,pass,sizeof(pass),stoptime) ) {
		if(passfile) {
			if(auth_userpass_lookup(name,pass)) {
				debug("auth_userpass: client authenticates as %s",name);
				memset(pass,0,strlen(pass));
				*subject = strdup(name);
				success = 1;
			} else {
				debug("auth_userpass: client could not authenticate");
			}
		} else {
			debug("auth_userpass: no password file defined!");
		}
	} else {
		debug("auth_userpass: lost connection");
	}

	if(success) {
		link_write(link,"yes\n",4,stoptime);
	} else {
		sleep(5);
		debug("auth_userpass: permission denied");
		link_write(link,"no\n",3,stoptime);
	}
	
	return success;
}

int auth_userpass_register( const char *p )
{
	passfile = p;
	debug("auth_userpass: registered");
	return auth_register( "userpass", auth_userpass_assert, auth_userpass_accept );
}

