
#include "generic_file.h"

#include "sparse_file.h"
#include "full_io.h"
#include "delete_dir.h"

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

struct generic_file {
	generic_file_t type;
	struct sparse_file *s;
	int f;
};

struct generic_file * generic_file_open( const char *filename, generic_file_t type )
{
	struct generic_file *g;

	g = malloc(sizeof(*g));
	if(!g) return 0;

	g->type = type;

	if(type==GENERIC_FILE_PLAIN) {
		g->f = open(filename,O_RDONLY);
		if(g->f<0) {
			free(g);
			return 0;
		}
	} else if(type==GENERIC_FILE_SPARSE) {
		g->s = sparse_file_open(filename);
		if(!g->s) {
			free(g);
			return 0;
		}
	} else {
		return 0;
	}

	return g;
}

struct generic_file * generic_file_create( const char *filename, generic_file_t type )
{
	struct generic_file *g;

	g = malloc(sizeof(*g));
	if(!g) return 0;

	g->type = type;

	if(type==GENERIC_FILE_PLAIN) {
		g->f = creat(filename,0777);
		if(g->f<0) {
			free(g);
			return 0;
		}
	} else {
		g->s = sparse_file_create(filename);
		if(!g->s) {
			free(g);
			return 0;
		}
	}

	return g;
}

void generic_file_close( struct generic_file *g )
{
	if(g->type==GENERIC_FILE_PLAIN) {
		close(g->f);
	} else {
		sparse_file_close(g->s);
	}

	free(g);
}

void generic_file_delete( const char *filename )
{
	delete_dir(filename);
}

int generic_file_commit( struct generic_file *g )
{
	int result;

	if(g->type==GENERIC_FILE_PLAIN) {
		result = !fsync(g->f);
	} else {
		result = sparse_file_commit(g->s);		
	}

	return result;
}

ssize_t generic_file_read( struct generic_file *g, void *data, size_t length, size_t offset )
{
	sparse_file_length_t actual;

	if(g->type==GENERIC_FILE_PLAIN) {
		return full_pread(g->f,data,length,offset);
	} else {
		if(sparse_file_read(g->s,data,length,offset,&actual)) {
			return actual;
		} else {
			return -1;
		}
	}
}

ssize_t generic_file_write( struct generic_file *g, const void *data, size_t length, size_t offset )
{
	if(g->type==GENERIC_FILE_PLAIN) {
		return full_pwrite(g->f,data,length,offset);
	} else {
		if(sparse_file_write(g->s,data,length,offset)) {
			return length;
		} else {
			return -1;
		}
	}
}


