/*
dttools is Copyright (C) 2002 Douglas Thain
This software is released under the GNU General Public License.
See the file COPYING for details.
*/


#include "multi_hash_table.h"
#include "hash_table.h"
#include "list.h"

#include <stdlib.h>

struct multi_hash_table {
	struct hash_table *table;
};

struct multi_hash_table * multi_hash_table_create( int bucket_count, hash_func_t func )
{
	struct multi_hash_table *h;

	h = malloc(sizeof(*h));
	if(!h) return 0;

	h->table = hash_table_create(bucket_count,func);
	if(!h->table) {
		free(h);
		return 0;
	}

	return h;
}

void multi_hash_table_delete( struct multi_hash_table *h )
{
	if(h) {
		hash_table_delete(h->table);
		free(h);
	}
}

void * multi_hash_table_peek_head( struct multi_hash_table *h, const char *key )
{
	struct list *list;

	list = hash_table_lookup(h->table,key);
	if(!list) return 0;

	return list_peek_head(list);
}

int multi_hash_table_push_head( struct multi_hash_table *h, const char *key, const void *value )
{
	struct list *list;
	int result;

	list = hash_table_lookup(h->table,key);
	if(!list) {
		list = list_create();
		if(!list) return 0;

		if(!hash_table_insert(h->table,key,list,0)) {
			list_delete(list);
			return 0;
		}
	}

	result = list_push_head(list,(void*)value);

	return result;
}

void * multi_hash_table_pop_head( struct multi_hash_table *h, const char *key )
{
	struct list *list;
	void * result;

	list = hash_table_lookup(h->table,key);
	if(!list) return 0;

	result = list_pop_head(list);
	return result;
}

void * multi_hash_table_peek_tail( struct multi_hash_table *h, const char *key )
{
	struct list *list;

	list = hash_table_lookup(h->table,key);
	if(!list) return 0;

	return list_peek_tail(list);
}

int multi_hash_table_push_tail( struct multi_hash_table *h, const char *key, const void *value )
{
	struct list *list;
	int result;

	list = hash_table_lookup(h->table,key);
	if(!list) {
		list = list_create();
		if(!list) return 0;

		if(!hash_table_insert(h->table,key,list,0)) {
			list_delete(list);
			return 0;
		}
	}

	result = list_push_tail(list,(void*)value);

	return result;
}

void * multi_hash_table_pop_tail( struct multi_hash_table *h, const char *key )
{
	struct list *list;
	void * result;

	list = hash_table_lookup(h->table,key);
	if(!list) return 0;

	result = list_pop_tail(list);
	return result;
}
